"use strict";

var TimeoutError = require("./TimeoutError");

module.exports = function timeout(ms, onReject) {
  var _this = this;

  if (ms === 0) {
    return this;
  }

  if (onReject === undefined) {
    onReject = new TimeoutError();
  }

  return new Promise(function (resolve, reject) {
    var handle = setTimeout(function () {
      handle = undefined;

      if (typeof _this.cancel === "function") {
        _this.cancel();
      }

      if (typeof onReject === "function") {
        try {
          resolve(onReject());
        } catch (error) {
          reject(error);
        }
      } else {
        reject(onReject);
      }
    }, ms);

    _this.then(function (value) {
      handle !== undefined && clearTimeout(handle);
      resolve(value);
    }, function (reason) {
      handle !== undefined && clearTimeout(handle);
      reject(reason);
    });
  });
};